'use strict';

var path = require('path');
var helperModuleImports = require('@babel/helper-module-imports');

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

var NamingRule = {
  LITTLE_CAMEL: 'little-camel',
  BIG_CAMEL: 'big-camel',
  UNDERLINE: 'underline',
  DASH: 'dash',
  FUNCTION: 'FUNCTION'
};

/**
 * @desc 封装了一些项目常用方法.
 */
// 内部函数, 用于判断对象类型
function _getClass(object) {
  return Object.prototype.toString.call(object).match(/^\[object\s(.*)\]$/)[1];
}

function isArray(obj) {
  return _getClass(obj).toLowerCase() === 'array';
}
function isString(obj) {
  return _getClass(obj).toLowerCase() === 'string';
}
function isObject(obj) {
  return _getClass(obj).toLowerCase() === 'object';
}
function isNumber(obj) {
  return _getClass(obj).toLowerCase() === 'number';
}
function isFunction(obj) {
  return _getClass(obj).toLowerCase() === 'function';
}
/**
 * @desc 判断参数是否为空, 包括null, undefined, [], '', {}
 * @param {object} obj 需判断的对象
 */

function isEmpty(obj) {
  var empty = false;

  if (obj === null || obj === undefined) {
    // null and undefined
    empty = true;
  } else if ((isArray(obj) || isString(obj)) && obj.length === 0) {
    empty = true;
  } else if (isObject(obj)) {
    var hasProp = false;

    for (var prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        hasProp = true;
        break;
      }
    }

    if (!hasProp) {
      empty = true;
    }
  } else if (isNumber(obj) && isNaN(obj)) {
    empty = true;
  }

  return empty;
}
/**
 * @desc 判断参数是否为空字符串, 比isEmpty()多判断字符串中全是空格的情况, 如: '   '.
 * @param {string} str 需判断的字符串
 */

function isBlank(str) {
  if (isEmpty(str)) {
    return true;
  } else if (isString(str) && str.trim().length === 0) {
    return true;
  }

  return false;
}
/**
 * @desc 判断参数是否不为空字符串
 */

function isNotBlank(obj) {
  return !isBlank(obj);
}

function toLittleCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[A-Z]/, function (match) {
    return match.toLowerCase();
  });
}
function toBigCamel() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/^[a-z]/, function (match) {
    return match.toUpperCase();
  });
}
function toDash() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '-' + _str : _str;
  });
}
function toUnderline() {
  var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return str.replace(/[A-Z]/g, function (match, offset) {
    var _str = match.toLowerCase();

    return offset > 0 ? '_' + _str : _str;
  });
}

var _RuleHandler;
var toLittleCamel$1 = toLittleCamel,
    toBigCamel$1 = toBigCamel,
    toDash$1 = toDash,
    toUnderline$1 = toUnderline;
var LITTLE_CAMEL = NamingRule.LITTLE_CAMEL,
    BIG_CAMEL = NamingRule.BIG_CAMEL,
    DASH = NamingRule.DASH,
    UNDERLINE = NamingRule.UNDERLINE;
var RuleHandler = (_RuleHandler = {}, _defineProperty(_RuleHandler, LITTLE_CAMEL, toLittleCamel$1), _defineProperty(_RuleHandler, BIG_CAMEL, toBigCamel$1), _defineProperty(_RuleHandler, DASH, toDash$1), _defineProperty(_RuleHandler, UNDERLINE, toUnderline$1), _RuleHandler);

function buildNameByRule(name, rule) {
  var ruleName;

  if (RuleHandler[rule]) {
    ruleName = RuleHandler[rule](name);
  } else if (isFunction(rule)) {
    ruleName = buildNameByRule(name, rule(name)); // 可能 function 返回 'little-camel'等规则
  } else if (isString(rule)) {
    // 明确的命名
    ruleName = rule.trim();
  } else {
    ruleName = name;
  }
  /*   switch (rule) {
      case NamingRule.LITTLE_CAMEL:   // 小驼峰
          name = toLittleCamel(name);
          break;
      case NamingRule.BIG_CAMEL:      // 大驼峰
          name = toBigCamel(name);
          break;
      case NamingRule.DASH:           // 中横线
          name = toDash(name);
          break;
      case NamingRule.UNDERLINE:     // 下划线
          name = toUnderline(name);
          break;
      default:
          if (isString(rule)) {       // specific name
              name = rule.trim();
          } else if (isFunction(rule)) {
              name = rule(name, helpers);
              if(){
                }
          }
  } */


  return ruleName;
}

function buildPath() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var paths = args.map(function (path) {
    return path || '';
  });
  return path.join.apply(void 0, _toConsumableArray(paths)).replace(/\\/g, '/');
}

var Plugin =
/*#__PURE__*/
function () {
  function Plugin(_ref) {
    var types = _ref.types;

    _classCallCheck(this, Plugin);

    _defineProperty(this, "_options", void 0);

    _defineProperty(this, "types", void 0);

    _defineProperty(this, "removePaths", void 0);

    _defineProperty(this, "markIdentifiers", void 0);

    this.types = types;
  }

  _createClass(Plugin, [{
    key: "setOptions",
    value: function setOptions(options) {
      // default setting    
      this._options = Object.assign({
        directory: null,
        namingRule: NamingRule.BIG_CAMEL,
        importDefault: true,
        style: false
      }, options);
    }
  }, {
    key: "init",
    value: function init() {
      this.removePaths = [];
      this.markIdentifiers = {};
    }
  }, {
    key: "cleanup",
    value: function cleanup() {
      this.removePaths.forEach(function (path) {
        return path.remove();
      });
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this._options = null; // 这两项设为空, 项目使用时会报缓存问题: api.cache(true).

      /* 
      this.types = null;
      this.replace = null;
          */
    }
  }, {
    key: "replaceIdentifierNodeName",
    value: function replaceIdentifierNodeName(node, path) {
      var nodeName = node.name;
      var newIdentifier = this.markIdentifiers[nodeName]; // { type: 'Identifier', name: '_default4' }

      if (newIdentifier && path.scope.hasBinding(nodeName) && path.scope.getBinding(nodeName).path.type === 'ImportSpecifier') {
        // 文件中可能有重复的变量名, 确保类型是 ImportSpecifier
        node.name = newIdentifier.name;
      }
    }
  }, {
    key: "getReplaceIdentifierNode",
    value: function getReplaceIdentifierNode(node) {
      var identifier;

      if (!node) {
        return;
      }

      switch (node.type) {
        case 'Identifier':
          identifier = node;
          break;

        case 'MemberExpression':
          identifier = this.getReplaceIdentifierNode(node.object);
          break;

        case 'CallExpression':
          identifier = this.getReplaceIdentifierNode(node.callee);
          break;
      }

      return identifier;
    }
  }, {
    key: "replaceIdentifier",
    value: function replaceIdentifier(node, path) {
      var identifierNode = this.getReplaceIdentifierNode(node);

      if (identifierNode) {
        this.replaceIdentifierNodeName(identifierNode, path);
      }
    } // 根据大写字母开头的方法来筛选 Visitor

  }, {
    key: "getVisitor",
    value: function getVisitor() {
      var _this = this;

      var visitor = {};
      var proto = Object.getPrototypeOf(this);
      Object.getOwnPropertyNames(proto).forEach(function (key) {
        var prop = _this[key];

        if (key.match(/^[A-Z]/) && typeof prop === 'function') {
          // 注意: 这里把 this 指向了 Plugin 类的实例
          visitor[key] = prop.bind(_this);
        }
      });
      return visitor;
    }
    /* 
    TODO: getVisitors 扩展对象
    Program = {
        enter(path, { opts = {} }) {
            plugin.init(opts);
        },
        exit(path, { opts = {} }) {
            plugin && plugin.destroy();
        }
    } */

    /**
     * import React from 'react';
     * import { Steps, DatePicker } from 'antd';
     * import antd, { Button as B, Input } from 'antd';
     */

  }, {
    key: "ImportDeclaration",
    value: function ImportDeclaration(path, state) {
      var _this2 = this;

      if (!path.node) {
        return;
      }

      var source = path.node.source.value;
      var _this$_options = this._options,
          library = _this$_options.library,
          directory = _this$_options.directory,
          namingRule = _this$_options.namingRule,
          importDefault = _this$_options.importDefault,
          style = _this$_options.style;

      if (source === library) {
        path.node.specifiers.forEach(function (specifier) {
          if (_this2.types.isImportSpecifier(specifier)) {
            var imported = specifier.imported,
                local = specifier.local; // 处理组件的命名规则(folder or file)

            var componentName = buildNameByRule(imported.name, namingRule); // 处理组件的全路径, /library/directory/componentName

            var componentPath = buildPath(library, directory, componentName);
            var newImportDeclaration; // 添加修改过路径的新 JS 节点

            if (importDefault) {
              newImportDeclaration = helperModuleImports.addDefault(path, componentPath); // import hintedName from "source"
            } else {
              newImportDeclaration = helperModuleImports.addNamed(path, local.name, componentPath); // import { named } from "source"
            } // 保存需要替换的标识符 Identifier 名称, 注意: 可能会和其他地方的局部变量标识符重复, 确保path.type是ImportSpecifier


            _this2.markIdentifiers[local.name] = newImportDeclaration; // handle style node

            if (style) {
              var _directory = style.directory,
                  _namingRule = style.namingRule,
                  ext = style.ext; // style 路径默认使用 /library/directory

              var styleDirectory = buildPath(library, directory); // set default directory

              var styleName = '';
              var extension = '';

              if (_directory) {
                if (_directory.startsWith('/')) {
                  // 从根路径开始设置样式路径, 说明样式文件保存在和组件不同的目录, 如: '/less/component' to '/library/less/component'.
                  styleDirectory = buildPath(library, _directory);
                } else {
                  // 在 componentPath 后追加路径, 说明样式文件保存在组件下的目录, 如: 'style' to '/library/directory/componentName/style'.
                  styleDirectory = buildPath(componentPath, _directory);
                }
              }

              if (_namingRule) {
                // 处理style的命名规则(folder or file)
                styleName = buildNameByRule(imported.name, _namingRule);
              }

              if (isNotBlank(ext)) {
                extension = ext.trim().startsWith('.') ? ext : '.' + ext;
              } // 处理style的全路径


              var stylePath = buildPath(styleDirectory, styleName + extension); // 添加 style 节点

              helperModuleImports.addSideEffect(path, stylePath);
            }
          }
        });
        this.removePaths.push(path);
      }
    }
    /** 
     *  {foo} in import {foo} from "mod" 
     *  {foo as bar} in import {foo as bar} from "mod" 
     */

  }, {
    key: "ImportSpecifier",
    value: function ImportSpecifier(path, state) {}
    /**
     * "foo" in import foo from 'mod'.
     */

  }, {
    key: "ImportDefaultSpecifier",
    value: function ImportDefaultSpecifier(path, state) {}
    /**
     * "* as foo" in import * as foo from 'mod'.
     */

  }, {
    key: "ImportNamespaceSpecifier",
    value: function ImportNamespaceSpecifier(path, state) {} // 粒度太小, 使用更大级别的节点类型

    /* 
        dentifier(path, state) {}
     */
    // <Component></Component>

  }, {
    key: "JSXIdentifier",
    value: function JSXIdentifier(path, state) {
      // 替换JSX中使用到的标识符名称, 确保一致.
      this.replaceIdentifierNodeName(path.node, path);
    }
  }, {
    key: "MemberExpression",
    value: function MemberExpression(path, state) {} // TODO: 

    /* 
        var Form1 = Form;
        var Item = Form.Item; 
        var { Item } = Form; 
    */

  }, {
    key: "VariableDeclarator",
    value: function VariableDeclarator(path, state) {
      this.replaceIdentifier(path.node.init, path);
    }
    /*
        Form1 = Form;
        Item = Form.item;
     */

  }, {
    key: "ExpressionStatement",
    value: function ExpressionStatement(path, state) {// TODO: type = AssignmentExpression.right = Identifier | MemberExpression
    }
  }, {
    key: "AssignmentExpression",
    value: function AssignmentExpression(path, state) {
      this.replaceIdentifier(path.node.right, path);
    }
  }, {
    key: "CallExpression",
    value: function CallExpression(path, state) {
      this.replaceIdentifier(path.node.callee, path);
    }
  }, {
    key: "ReturnStatement",
    value: function ReturnStatement(path, state) {}
  }]);

  return Plugin;
}();

function index (_ref) {
  var types = _ref.types;
  // 此 this 是 babel-plugin 环境
  var plugin = new Plugin({
    types: types
  });
  return {
    pre: function pre() {
      plugin.setOptions(this.opts);
    },
    visitor: _objectSpread({
      Program: {
        enter: function enter(path, _ref2) {
          var opts = _ref2.opts;
          plugin.init();
        },
        exit: function exit(path, _ref3) {
          var opts = _ref3.opts;
          plugin.cleanup();
        }
      }
    }, plugin.getVisitor()),
    post: function post() {
      plugin.destroy();
    }
  };
}

module.exports = index;
